@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Library::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Library::VERSION   = '1.007_014';
+	$Type::Library::VERSION   = '1.009_001';
 }
 
 $Type::Library::VERSION =~ tr/_//d;
@@ -239,13 +239,17 @@ sub _exporter_fail
 		{
 			my $params; $params = shift if ref($_[0]) eq "ARRAY";
 			my $type = $into->get_type($name);
-			unless ($type)
+			my $t;
+			
+			if ($type) {
+				my $t = $params ? $type->parameterize(@$params) : $type;
+			}
+			else
 			{
 				_croak "Cannot parameterize a non-existant type" if $params;
-				$type = $name;
+				$t = Type::Tiny::_DeclaredType->new(library => $into, name => $name);
 			}
 			
-			my $t = $params ? $type->parameterize(@$params) : $type;
 			@_ && wantarray ? return($t, @_) : return $t;
 		};
 		
@@ -261,6 +265,27 @@ sub _exporter_fail
 	return $class->SUPER::_exporter_fail(@_);
 }
 
+{
+	package Type::Tiny::_DeclaredType;
+	our @ISA = 'Type::Tiny';
+	sub new {
+		my $class = shift;
+		my %opts  = @_==1 ? %{+shift} : @_;
+		my $library = delete $opts{library};
+		my $name    = delete $opts{name};
+		$opts{display_name} = $name;
+		$opts{constraint} = sub {
+			my $val = @_ ? pop : $_;
+			$library->get_type($name)->check($val);
+		};
+		$opts{inlined} = sub {
+			my $val = @_ ? pop : $_;
+			sprintf('%s::is_%s(%s)', $library, $name, $val);
+		};
+		$class->SUPER::new(%opts);
+	}
+}
+
 sub meta
 {
 	no strict "refs";
@@ -645,7 +670,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
