@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Enum::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Enum::VERSION   = '1.007_014';
+	$Type::Tiny::Enum::VERSION   = '1.009_001';
 }
 
 $Type::Tiny::Enum::VERSION =~ tr/_//d;
@@ -68,7 +68,8 @@ sub _build_display_name
 	{
 		my $self = shift;
 		
-		my $regexp  = join "|", map quotemeta, @{$self->unique_values};
+		my $regexp  = join "|", map quotemeta, sort {length $b <=> length $a || $a cmp $b } @{$self->unique_values};
+		$regexp = '(?!)' unless @{$self->unique_values};
 		return $cached{$regexp} if $cached{$regexp};
 		my $coderef = ($cached{$regexp} = sub { defined and m{\A(?:$regexp)\z} });
 		Scalar::Util::weaken($cached{$regexp});
@@ -81,7 +82,8 @@ sub _build_display_name
 	sub _build_compiled_check
 	{
 		my $self = shift;
-		my $regexp  = join "|", map quotemeta, @{$self->unique_values};
+		my $regexp  = join "|", map quotemeta, sort {length $b <=> length $a || $a cmp $b } @{$self->unique_values};
+		$regexp = '(?!)' unless @{$self->unique_values};
 		return $cached{$regexp} if $cached{$regexp};
 		my $coderef = ($cached{$regexp} = $self->SUPER::_build_compiled_check(@_));
 		Scalar::Util::weaken($cached{$regexp});
@@ -89,6 +91,21 @@ sub _build_display_name
 	}
 }
 
+sub as_regexp
+{
+	my $self = shift;
+	
+	my $flags = @_ ? $_[0] : '';
+	unless (defined $flags and $flags =~ /^[i]*$/) {
+		_croak("Unknown regexp flags: '$flags'; only 'i' currently accepted; stopped");
+	}
+	
+	my $regexp  = join "|", map quotemeta, sort {length $b <=> length $a || $a cmp $b } @{$self->unique_values};
+	$regexp = '(?!)' unless @{$self->unique_values};
+	
+	$flags ? qr/\A(?:$regexp)\z/i : qr/\A(?:$regexp)\z/;
+}
+
 sub can_be_inlined
 {
 	!!1;
@@ -245,6 +262,33 @@ be passed to the constructor.
 
 =back
 
+=head2 Methods
+
+=over
+
+=item C<as_regexp>
+
+Returns the enum as a regexp which strings can be checked against. If you're
+checking I<< a lot >> of strings, then using this regexp might be faster than
+checking each string against 
+
+  my $enum  = Type::Tiny::Enum->new(...);
+  my $check = $enum->compiled_check;
+  my $re    = $enum->as_regexp;
+  
+  # fast
+  my @valid_tokens = grep $enum->check($_), @all_tokens;
+  
+  # faster
+  my @valid_tokens = grep $check->($_), @all_tokens;
+  
+  # fastest
+  my @valid_tokens = grep /$re/, @all_tokens;
+
+You can get a case-insensitive regexp using C<< $enum->as_regexp('i') >>.
+
+=back
+
 =head2 Overloading
 
 =over
@@ -274,7 +318,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
