@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Tiny::Role::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Role::VERSION   = '1.007_014';
+	$Type::Tiny::Role::VERSION   = '1.009_001';
 }
 
 $Type::Tiny::Role::VERSION =~ tr/_//d;
@@ -15,7 +15,7 @@ use Scalar::Util qw< blessed weaken >;
 
 sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-require Type::Tiny::ConstrainedObject;
+use Type::Tiny::ConstrainedObject ();
 our @ISA = 'Type::Tiny::ConstrainedObject';
 sub _short_name { 'Role' }
 
@@ -169,7 +169,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
