@@ -12,7 +12,7 @@ BEGIN {
 
 BEGIN {
 	$Types::Standard::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::VERSION   = '1.007_014';
+	$Types::Standard::VERSION   = '1.009_001';
 }
 
 $Types::Standard::VERSION =~ tr/_//d;
@@ -1319,7 +1319,7 @@ Here's an example using L<Regexp::Common>:
       use Regexp::Common;
       has ip_address => (
          is         => 'ro',
-         required   => 1.
+         required   => 1,
          isa        => StrMatch[/^$RE{net}{IPv4}$/],
          default    => '127.0.0.1',
       );
@@ -1533,7 +1533,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
