@@ -4,13 +4,21 @@ use strict;
 use warnings;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '1.007_014';
+our $VERSION   = '1.009_001';
 
 $VERSION =~ tr/_//d;
 
 use Scalar::Util qw< blessed refaddr weaken >;
 
-our @EXPORT_OK = ( __PACKAGE__->type_names, qw/to_TypeTiny/ );
+our @EXPORT_OK = (
+	map(@{[ $_, "is_$_", "assert_$_" ]}, __PACKAGE__->type_names),
+	qw/to_TypeTiny/
+);
+our %EXPORT_TAGS = (
+	types     => [                  __PACKAGE__->type_names ],
+	is        => [ map "is_$_",     __PACKAGE__->type_names ],
+	assert    => [ map "assert_$_", __PACKAGE__->type_names ],
+);
 
 my %cache;
 
@@ -34,6 +42,13 @@ sub import
 	return $class->$next($opts, @_);              # uncoverable statement
 }
 
+for (__PACKAGE__->type_names) {
+	eval qq{
+		sub is_$_     { $_()->check(shift) }
+		sub assert_$_ { $_()->assert_return(shift) }
+	};
+}
+
 sub meta
 {
 	return $_[0];
@@ -115,13 +130,20 @@ sub _check_overload
 	!!0;
 }
 
+sub _get_check_overload_sub {
+	if ($Type::Tiny::AvoidCallbacks) {
+		return '(sub { require overload; overload::Overloaded(ref $_[0] or $_[0]) and overload::Method((ref $_[0] or $_[0]), $_[1]) })->'
+	}
+	return 'Types::TypeTiny::_check_overload';
+}
+
 sub StringLike ()
 {
 	require Type::Tiny;
 	$cache{StringLike} ||= "Type::Tiny"->new(
 		name       => "StringLike",
 		constraint => sub {    defined($_   ) && !ref($_   ) or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[""])  },
-		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && Types::TypeTiny::_check_overload($_[1], q[""])/ },
+		inlined    => sub { qq/defined($_[1]) && !ref($_[1]) or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[""])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -132,7 +154,7 @@ sub HashLike ()
 	$cache{HashLike} ||= "Type::Tiny"->new(
 		name       => "HashLike",
 		constraint => sub {    ref($_   ) eq q[HASH] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[%{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && Types::TypeTiny::_check_overload($_[1], q[\%{}])/ },
+		inlined    => sub { qq/ref($_[1]) eq q[HASH] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\%{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -143,7 +165,7 @@ sub ArrayLike ()
 	$cache{ArrayLike} ||= "Type::Tiny"->new(
 		name       => "ArrayLike",
 		constraint => sub {    ref($_   ) eq q[ARRAY] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[@{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && Types::TypeTiny::_check_overload($_[1], q[\@{}])/ },
+		inlined    => sub { qq/ref($_[1]) eq q[ARRAY] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\@{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -154,7 +176,7 @@ sub CodeLike ()
 	$cache{CodeLike} ||= "Type::Tiny"->new(
 		name       => "CodeLike",
 		constraint => sub {    ref($_   ) eq q[CODE] or Scalar::Util::blessed($_   ) && Types::TypeTiny::_check_overload($_   , q[&{}])  },
-		inlined    => sub { qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && Types::TypeTiny::_check_overload($_[1], q[\&{}])/ },
+		inlined    => sub { qq/ref($_[1]) eq q[CODE] or Scalar::Util::blessed($_[1]) && ${\ +_get_check_overload_sub() }($_[1], q[\&{}])/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -181,7 +203,7 @@ sub _ForeignTypeConstraint ()
 	$cache{_ForeignTypeConstraint} ||= "Type::Tiny"->new(
 		name       => "_ForeignTypeConstraint",
 		constraint => \&_is_ForeignTypeConstraint,
-		inlined    => sub { qq/ref($_[1]) && Types::TypeTiny::_is_ForeignTypeConstraint($_[1])/ },
+		inlined    => sub { qq/ref($_[1]) && do { require Types::TypeTiny; Types::TypeTiny::_is_ForeignTypeConstraint($_[1]) }/ },
 		library    => __PACKAGE__,
 	);
 }
@@ -246,25 +268,22 @@ sub _TypeTinyFromMoose
 		return $ts if $ts->{_is_core};
 	}
 	
-	my %opts;
-	$opts{display_name} = $t->name;
-	$opts{constraint}   = $t->constraint;
-	$opts{parent}       = to_TypeTiny($t->parent)              if $t->has_parent;
-	$opts{inlined}      = sub { shift; $t->_inline_check(@_) } if $t->can("can_be_inlined") && $t->can_be_inlined;
-	$opts{message}      = sub { $t->get_message($_) }          if $t->has_message;
-	$opts{moose_type}   = $t;
+	my ($tt_class, $tt_opts) =
+		$t->can('parameterize')                          ? _TypeTinyFromMoose_parameterizable($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Enum')     ? _TypeTinyFromMoose_enum($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Class')    ? _TypeTinyFromMoose_class($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Role')     ? _TypeTinyFromMoose_role($t) :
+		$t->isa('Moose::Meta::TypeConstraint::Union')    ? _TypeTinyFromMoose_union($t) :
+		$t->isa('Moose::Meta::TypeConstraint::DuckType') ? _TypeTinyFromMoose_ducktype($t) :
+		_TypeTinyFromMoose_baseclass($t);
 	
-	if ($t->can('parameterize')) {
-		$opts{constraint_generator} = sub {
-			# convert args into Moose native types; not strictly necessary
-			my @args    = map { TypeTiny->check($_) ? $_->moose_type : $_ } @_;
-			my $paramd  = $t->parameterize(@args);
-			_TypeTinyFromMoose($paramd);
-		};
-	}
-
-	require Type::Tiny;
-	my $new = 'Type::Tiny'->new(%opts);
+	# Standard stuff to do with all type constraints from Moose,
+	# regardless of variety.
+	$tt_opts->{moose_type}   = $t;
+	$tt_opts->{display_name} = $t->name;
+	$tt_opts->{message}      = sub { $t->get_message($_) } if $t->has_message;
+	
+	my $new = $tt_class->new(%$tt_opts);
 	$ttt_cache{ refaddr($t) } = $new;
 	weaken($ttt_cache{ refaddr($t) });
 	
@@ -279,6 +298,71 @@ sub _TypeTinyFromMoose
 	return $new;
 }
 
+sub _TypeTinyFromMoose_baseclass
+{
+	my $t = shift;
+	my %opts;
+	$opts{parent}       = to_TypeTiny($t->parent)              if $t->has_parent;
+	$opts{constraint}   = $t->constraint;
+	$opts{inlined}      = sub { shift; $t->_inline_check(@_) } if $t->can("can_be_inlined") && $t->can_be_inlined;
+	
+	# Cowardly refuse to inline types that need to close over stuff
+	if ($opts{inlined}) {
+		my %env = %{ $t->inline_environment || {} };
+		delete($opts{inlined}) if keys %env;
+	}
+	
+	require Type::Tiny;
+	return 'Type::Tiny' => \%opts;
+}
+
+sub _TypeTinyFromMoose_union
+{
+	my $t = shift;
+	require Type::Tiny::Union;
+	return 'Type::Tiny::Union' => { type_constraints => [ map _TypeTinyFromMoose($_), @{$t->type_constraints} ] };
+}
+
+sub _TypeTinyFromMoose_enum
+{
+	my $t = shift;
+	require Type::Tiny::Enum;
+	return 'Type::Tiny::Enum' => { values => [@{ $t->values }] };
+}
+
+sub _TypeTinyFromMoose_class
+{
+	my $t = shift;
+	require Type::Tiny::Class;
+	return 'Type::Tiny::Class' => { class => $t->class };
+}
+
+sub _TypeTinyFromMoose_role
+{
+	my $t = shift;
+	require Type::Tiny::Role;
+	return 'Type::Tiny::Role' => { role => $t->role };
+}
+
+sub _TypeTinyFromMoose_ducktype
+{
+	my $t = shift;
+	require Type::Tiny::Duck;
+	return 'Type::Tiny::Duck' => { methods => [@{ $t->methods }] };
+}
+
+sub _TypeTinyFromMoose_parameterizable
+{
+	my $t = shift;
+	my ($class, $opts) = _TypeTinyFromMoose_baseclass($t);
+	$opts->{constraint_generator} = sub {
+		# convert args into Moose native types; not strictly necessary
+		my @args = map { TypeTiny->check($_) ? $_->moose_type : $_ } @_;
+		_TypeTinyFromMoose( $t->parameterize(@args) );
+	};
+	return ($class, $opts);
+}
+
 sub _TypeTinyFromValidationClass
 {
 	my $t = $_[0];
@@ -391,8 +475,7 @@ sub _TypeTinyFromMouse
 		$opts{constraint_generator} = sub {
 			# convert args into Moose native types; not strictly necessary
 			my @args    = map { TypeTiny->check($_) ? $_->mouse_type : $_ } @_;
-			my $paramd  = $t->parameterize(@args);
-			_TypeTinyFromMouse($paramd);
+			_TypeTinyFromMouse( $t->parameterize(@args) );
 		};
 	}
 
@@ -569,7 +652,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2020 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
